
package Text::BibTeX::Filter;
use Text::BibTeX;
use Text::BibTeX::File;
use strict;
use Data::Dumper;
my $debug=1;
my %fields=(
	    check=>0,
	    strip_fields=>undef,
	    remap=>undef,
	    select=>undef
	   );
	    
sub new(){
  my $class=shift;
  my $filename=shift;
  my $bibstructure=shift;
  my $macros=shift;
  my %opts=@_;

  my $file=new  Text::BibTeX::File $filename;

  return undef if (!defined($file));

  # This is tacky. But otherwise I need to subclase T::B::File
  $file->{structure}=$bibstructure;
  my $self=+{%fields,file=>$file,macros=>$macros,structure=>$bibstructure};

  foreach my $key (keys %opts){
    if ((exists $fields{$key}) || ($key=~ m/^skip|min|max/)){
      $self->{$key}=$opts{$key};
    }
  }
  bless $self,ref($class) || $class;
  return $self;
}

sub DESTROY{
  shift->{file}->close;
}

sub read(){
  my $self=shift;
  my $bibfile=$self->{file};
  my $entry;
  my $bibstructure=$self->{structure};

 
  my %macro=%{$self->{macros}};

 ENTRY: while ($entry = new Text::BibTeX::Entry $bibfile)
    {
      next unless ($entry->parse_ok && $entry->metatype == BTE_REGULAR);
      $entry->check if ($self->{check});

      my $bibsec=$entry->get("bibsec");
      my $sec;
      my $secopts=undef;
      my $secsel=undef;
      if ($bibsec){
	$sec=$bibstructure->section($bibsec);
	$secopts=$sec->{OPTIONS} if $sec;
	$secsel=$sec->{SELECT} if $sec;
      }

      # is this a keeper?
      my $selectref=$secsel || $self->{select};
      if ($selectref){

	my %select=%{$selectref};

	foreach my $key (keys %select){
	  my $compareto;
	  if ($key eq "cite"){
	    $compareto=$entry->key();
	  } else {
	    $compareto=$entry->get($key);
	  }
	  my $rex='^('.join("|",@{$select{$key}}).')$';

	  next ENTRY if (!($compareto=~m/$rex/));
	}
      }

      my %keyhash=%{$self};
      if (defined($secopts)){
	%keyhash=(%keyhash,%{$secopts});
#	print STDERR "$bibsec keyhash=",Dumper(\%keyhash),"\n";
      };

	  foreach my $key (keys %keyhash){
	    if ($key=~ m/^max_|min_/){
	  my ($dir,$field)=split("_",$key);
	  if ($entry->exists($field)){
	    my $endval=$entry->get($field);
	    my $startval=$entry->get($field);

	    if  ($field eq 'year'){
	      $endval=$entry->get('endyear');
	      $endval=$entry->get('year') if (!defined($endval) || !($endval =~ m/^\d+$/));
	    }
	  
	    next ENTRY if (($dir eq "min" && $endval < $keyhash{$key}) ||
			   ( $dir eq "max" && $startval > $keyhash{$key}) );
	  }
	}
	if ($key=~ m/^skip_/){
	  my ($dir,$field)=split("_",$key);
	  #	    print STDERR "$dir,$field";
	  my $rex=join("|",@{$keyhash{$key}});
	  if ( $entry->exists($field) ) {
	    
	    my @tags=split(",",$entry->get($field));
	    if (grep(m/$rex/,@tags)){
	      next ENTRY;
	    }
	  }
	}
      }


      
      # apply macros to keys as well
      my $bibkey=$entry->key;
      if (defined($macro{$bibkey})){
	$entry->set_key($macro{$bibkey});
      }

      # and to types of entries
      my $bibtype=$entry->type;
      if (defined($macro{$bibtype})){
	$entry->set_type($macro{$bibtype});
      }
      

      if ($self->{strip_fields}){
	foreach my $todie (@{$self->{strip_fields}}){
	  $entry->delete($todie);
	}
      }
	
      if ($self->{remap}){
	my %remap=@{$self->{remap}};
 	foreach my $field (keys %remap){
	  if ($entry->exists($field)){
	    my $newval=$entry->get($field);
#	    print STDERR "remapping $field = $newval \n" if $debug;
	    $entry->set($remap{$field}, $newval) ;
	  }

 	}
      }

 
      if ($sec) {
	  
#	print STDERR "checking ",$entry->key,"\n";
	next ENTRY  if $sec->collected($entry);
#	print STDERR "keeping ",$entry->key,"\n";	
	$entry->set_section($sec);
	$sec->add_entry($entry);
      }
      

      return $entry;

    }
  return undef;
}

1;
